module Functions.Pandoc
  ( readMarkdown
  , writeHtml
  , extractTitle
  ) where

import Types (Token)
import DependencyGenerator (DepGenM, TokenableTo(..), onToken)

import Data.Text (Text)
import Text.Pandoc.Definition (Pandoc)
import qualified Text.Pandoc.Definition as PD
import qualified Text.Pandoc.Shared as PS
import qualified Text.Pandoc as P

runPandoc :: P.PandocPure a -> a
runPandoc m = case P.runPure m of
  Left e -> error ("unexpected pandoc error: " ++ show e)
  Right result -> result

readMarkdown :: TokenableTo Text a => a -> DepGenM (Token Pandoc)
readMarkdown = onToken $ \contents ->
  let settings = P.def { P.readerExtensions = P.extensionsFromList [ P.Ext_raw_html ] }
  in runPandoc $ P.readMarkdown settings contents

writeHtml :: TokenableTo Pandoc a => a -> DepGenM (Token Text)
writeHtml = onToken $ \pandoc -> runPandoc $ P.writeHtml5String P.def pandoc

extractTitle :: TokenableTo Pandoc a => a -> DepGenM (Token Text)
extractTitle = onToken $ \(PD.Pandoc _ blocks) -> case blocks of
  (PD.Header 1 _ inlines : _) -> PS.stringify inlines
  _ -> error "unexpected"
