module Functions.Text
  ( concatStrings
  , concatTexts
  , lowerString
  , toText
  , readTextFile
  , saveTextFile
  ) where

import Types (IsFunction(..), IsFunctionIO(..), Token)
import DependencyGenerator (DepGenM, TokenableTo(..), toTupleToken,
                            runFunction, runFunctionIO, runFunctionIO_)

import Data.Char (toLower)
import Data.Text (Text)
import qualified Data.Text as T
import qualified Data.Text.IO as T


data ConcatStrings = ConcatStrings deriving Show
instance IsFunction ConcatStrings [String] String where
  evalFunction ConcatStrings = concat

concatStrings :: TokenableTo [String] a => a -> DepGenM (Token String)
concatStrings a = runFunction ConcatStrings =<< toToken a


data ConcatTexts = ConcatTexts deriving Show
instance IsFunction ConcatTexts [Text] Text where
  evalFunction ConcatTexts = T.concat

concatTexts :: TokenableTo [Text] a => a -> DepGenM (Token Text)
concatTexts a = runFunction ConcatTexts =<< toToken a


data LowerString = LowerString deriving Show
instance IsFunction LowerString String String where
  evalFunction LowerString s = map toLower s

lowerString :: TokenableTo String a => a -> DepGenM (Token String)
lowerString a = runFunction LowerString =<< toToken a


data ToText = ToText deriving Show
instance IsFunction ToText String Text where
  evalFunction ToText s = T.pack s

toText :: TokenableTo String a => a -> DepGenM (Token Text)
toText a = runFunction ToText =<< toToken a


data ReadTextFile = ReadTextFile deriving Show
instance IsFunctionIO ReadTextFile FilePath Text where
  evalFunctionIO ReadTextFile s =
    T.readFile s
  functionIOTouchesFilesystem ReadTextFile = False

readTextFile :: TokenableTo FilePath a => a -> DepGenM (Token Text)
readTextFile a = runFunctionIO ReadTextFile =<< toToken a


data SaveTextFile = SaveTextFile deriving Show
instance IsFunctionIO SaveTextFile (Text, FilePath) () where
  evalFunctionIO SaveTextFile (t, s) =
    T.writeFile s t
  functionIOTouchesFilesystem SaveTextFile = True

saveTextFile :: (TokenableTo Text a, TokenableTo FilePath b) => a -> b -> DepGenM ()
saveTextFile a b = runFunctionIO_ SaveTextFile =<< toTupleToken a b
